@@ -5,7 +5,7 @@
 pub use std::borrow::Borrow;
 pub use std::cmp::{max, Ordering};
 pub use std::convert::{TryFrom, TryInto};
-pub use std::fmt::{self, Debug, Display, Formatter};
+pub use std::fmt::{self, Debug, Display, Formatter, Write as _};
 pub use std::hash::{Hash, Hasher};
 pub use std::iter;
 pub use std::mem;
@@ -29,7 +29,7 @@ pub use void::{self, Void};
 pub use crate::html::*;
 
 pub use crate::geometry::{CoordinateOverflow, PosC, PosPromote};
-pub use crate::{hformat, hformat_as_display, hwrite};
+pub use crate::{dbgc, hformat, hformat_as_display, hwrite};
 pub use crate::misc::default;
 pub use crate::misc::display_as_debug;
 
