@@ -104,8 +104,12 @@ pub struct ZCoord(innards::Innards);
 pub struct ParseError;
 
 #[derive(Error,Clone,Copy,Debug,Eq,PartialEq,Serialize,Deserialize)]
-#[error("Z coordinate range has end before start, cannot iterate")]
-pub struct RangeBackwards;
+pub enum RangeImpossible {
+  #[error("Z coordinate range has end before start, cannot iterate")]
+  Backwards,
+  #[error("Z coordinate range has end equal to start, cannot iterate")]
+  Empty,
+}
 
 #[derive(Error,Clone,Copy,Debug,Eq,PartialEq,Serialize,Deserialize)]
 #[error("Z coordinate range has neither end, cannot iterate")]
@@ -118,7 +122,7 @@ pub struct Overflow;
 #[derive(Error,Clone,Copy,Debug,Eq,PartialEq,Serialize,Deserialize)]
 pub enum LogicError {
   #[error("{0}")] RangeTotallyUnbounded(#[from] TotallyUnboundedRange),
-  #[error("{0}")] RangeBackwards       (#[from] RangeBackwards       ),
+  #[error("{0}")] RangeImpossible      (#[from] RangeImpossible      ),
 }
 
 //---------- LimbVal ----------
@@ -365,12 +369,12 @@ impl Mutable {
     *self.limbs.get(i).unwrap_or(&ZERO)
   }
   fn extend_to_limb(&mut self, i: usize) {
-    if self.limbs.len() < i {
+    if self.limbs.len() <= i {
       self.limbs.resize(i+1, ZERO);
     }
   }
 
-  #[throws(RangeBackwards)]
+  #[throws(RangeImpossible)]
   fn range_core(a: &Mutable, b: &Mutable, count: RangeCount)
                 -> (Mutable, AddSubRangeDelta) {
     type ASRD = AddSubRangeDelta;
@@ -378,25 +382,33 @@ impl Mutable {
     let mut current = a.clone();
     let mut borrowing = false;
     let aso = 'ok: loop { for i in 0.. {
-      if i >= a.limbs.len() && i >= b.limbs.len() {
+      if i > a.limbs.len() && i > b.limbs.len() {
 	// Oh actually these numbers are equal!
-	break 'ok ASRD { i: 0, step: ZERO };
+        // (We have to tolerate one limb eyond each number, because possibly
+        // we had to borrow and add a limb to the longer number.)
+	throw!(RangeImpossible::Empty);
       }
+      //dbgc!(&a, &b, &count, i, &current, borrowing);
+
       current.extend_to_limb(i);
 
       let la = a.limb_val_lookup(i);
       let lb = b.limb_val_lookup(i);
-      if la == lb { continue }
+      if la == lb && ! borrowing {
+        // We have not yet found any difference between these numbers
+        // (If we had but it wasn't enough, `borrowing` would be `true`.
+        continue
+      }
 
       let wantgaps = count+1;
       let avail = (lb.primitive() as i64) - (la.primitive() as i64)
         + if borrowing { RAW_LIMB_MODULUS as i64 } else { 0};
-      if avail < 0 { throw!(RangeBackwards) }
+      if avail < 0 { throw!(RangeImpossible::Backwards) }
       let avail = avail as u64;
 
       if avail < 2 {
         // Only 1 difference in this limb and the next.  We are
-        // goint to have to borrow, and, later, add with carry.
+        // going to have to borrow, and, later, add with carry.
         borrowing = true;
         continue;
       }
@@ -426,7 +438,10 @@ impl Mutable {
     (current, aso)
   }
 
-  #[throws(RangeBackwards)]
+  #[throws(RangeImpossible)]
+  /// Iterator producing a half-open range `[self, other>`
+  ///
+  /// Produces precisely `count` items.
   pub fn range_upto(&self, other: &Mutable, count: RangeCount)
                     -> RangeIterator {
     let (current, aso) = Mutable::range_core(self, other, count)?;
@@ -451,12 +466,14 @@ pub type BoxedIterator = Box<dyn BoxedIteratorTrait>;
 impl<T> BoxedIteratorTrait for T where T: Iterator<Item = ZCoord> + Debug {}
 
 impl Mutable {
+  /// Iterator producing `<self, ..>`
   pub fn iter<ASO:AddSubOffset>(self, aso: ASO)
                                 -> IteratorCore<ASO, impl MutateReturn + Debug>
   {
     IteratorCore { current: self, aso, mr: MutateFirst }
   }
   #[throws(LogicError)]
+  /// Iterator producing an open range, `<a, b>`
   pub fn some_range(a: Option<&Mutable>, b: Option<&Mutable>,
                     count: RangeCount) -> BoxedIterator {
     fn mk<T:'static + Debug + Iterator<Item=ZCoord>>(x: T) -> BoxedIterator
@@ -768,9 +785,8 @@ mod test {
   use std::collections::hash_map::DefaultHasher;
   use std::mem;
 
-  fn bf(s: &str) -> ZCoord {
-    ZCoord::from_str(s).unwrap()
-  }
+  fn zc(s: &str) -> ZCoord { ZCoord::from_str(s).unwrap() }
+  fn mk(s: &str) -> super::Mutable { zc(s).clone_mut() }
 
   #[test]
   fn bfparse() {
@@ -819,16 +835,15 @@ mod test {
 
   #[test]
   fn inequality() {
-    assert!( bf("gg0123abcd_0123456789") <
-             bf("gg0123abcd_012345678a") );
+    assert!( zc("gg0123abcd_0123456789") <
+             zc("gg0123abcd_012345678a") );
     
-    assert!( bf("gg0123abcd") <
-             bf("gg0123abcd_012345678a") );
+    assert!( zc("gg0123abcd") <
+             zc("gg0123abcd_012345678a") );
   }
 
   #[test]
   fn incdec() {
-    fn mk(s: &str) -> super::Mutable { bf(s).clone_mut() }
     use core::cmp::Ordering::{Greater,Less};
     impl Mutable {
       fn tincdec<ASO:AddSubOffset>(mut self, exp: &str, aso: ASO,
@@ -882,6 +897,27 @@ mod test {
     mk("vvvvvvvvvv_vvvvvvvvvv_vvvvv01234")
       .tinc("vvvvvvvvvv_vvvvvvvvvv_vvvvv01234_0000000000_0001000000")
       ;
+
+    assert_eq!( Mutable::some_range(Some(&mk("0200000000")),
+                                    Some(&mk("0100000000")),
+                                    1).unwrap_err(),
+                LogicError::from(RangeImpossible::Backwards) );
+
+    assert_eq!( Mutable::some_range(Some(&mk("0200000000")),
+                                    Some(&mk("0200000000")),
+                                    1).unwrap_err(),
+                LogicError::from(RangeImpossible::Empty) );
+  }
+
+  #[test]
+  fn iter() {
+    let mut m = mk("000000000a").iter(Increment);
+    assert_eq!( m.next(), Some(zc("000100000a")) );
+    assert_eq!( m.next(), Some(zc("000200000a")) );
+
+    let mut m = mk("000000000a").iter(Decrement);
+    assert_eq!( m.next(), Some(zc("0000000009_vvvvvvvvvv_vvvuvvvvvv")) );
+    assert_eq!( m.next(), Some(zc("0000000009_vvvvvvvvvv_vvvtvvvvvv")) );
   }
 
   #[test]
@@ -891,24 +927,64 @@ mod test {
       last: ZCoord,
     }
     impl It {
+      fn new(x: &str, y: &str, count: u32) -> Self {
+        let x = mk(x);
+        let y = mk(y);
+        let i = x.range_upto(&y, count).unwrap();
+        It { i, last: x.repack().unwrap() }
+      }
       fn nxt(&mut self, exp: &str) {
         let got = self.i.next().unwrap();
         assert_eq!(got.to_string(), exp);
-        assert_eq!(got, bf(exp));
-        assert!(got > self.last);
+        assert_eq!(got, zc(exp));
+        assert!(got > self.last, "{:?} <= {:?} !", &got, &self.last);
         self.last = got.clone();
       }
     }
-    let x = bf("3333333333_vvvvvvvvv0").clone_mut();
-    let y = bf("3333333334_0000000040").clone_mut();
-    let i = x.range_upto(&y, 4).unwrap();
-    let mut it = It { i, last: x.repack().unwrap() };
+    let mut it = It::new("3333333333_vvvvvvvvv0",
+                         "3333333334_0000000040", 4);
     it.nxt("3333333334");
     it.nxt("3333333334_0000000010");
     it.nxt("3333333334_0000000020");
     it.nxt("3333333334_0000000030");
     assert_eq!(it.i.next(), None);
-  }
+
+    let mut it = It::new("3333333333_vvvvvvvvo0",
+                         "3333333334_0000000030", 4);
+    it.nxt("3333333333_vvvvvvvvq6");
+    it.nxt("3333333333_vvvvvvvvsc");
+    it.nxt("3333333333_vvvvvvvvui");
+    it.nxt("3333333334_000000000o");
+    assert_eq!(it.i.next(), None);
+
+    let mut it = It::new("aaaaaaaaaa_vvvvvvvvvv",
+                         "aaaaaaaaab"           , 2);
+    it.nxt("aaaaaaaaaa_vvvvvvvvvv_alalalalal");
+    it.nxt("aaaaaaaaaa_vvvvvvvvvv_lalalalala");
+    assert_eq!(it.i.next(), None);
+
+    let mut it = It::new("1000000000",
+                         "2000000000", 3);
+    it.nxt("1800000000");
+    it.nxt("1g00000000");
+    it.nxt("1o00000000");
+    assert_eq!(it.i.next(), None);
+
+    assert_eq!(mk             ("aaaaaaaaaa_vvvvvvvvvv")
+               .range_upto(&mk("aaaaaaaaaa_vvvvvvvvvv"), 4)
+               .unwrap_err(),
+               RangeImpossible::Empty);
+ 
+    assert_eq!(mk             ("0000000001")
+               .range_upto(&mk("0000000001"), 4)
+               .unwrap_err(),
+               RangeImpossible::Empty);
+
+    assert_eq!(mk             ("0000000002")
+               .range_upto(&mk("0000000001"), 4)
+               .unwrap_err(),
+               RangeImpossible::Backwards);
+ }
 
   #[test]
   fn some_range() {
@@ -931,7 +1007,7 @@ mod test {
         let got_s = got_s.as_ref().map(|s| s.as_str());
         assert_eq!(got_s, exp);
         if let (Some(got), Some(exp)) = (&got, &exp) {
-          assert_eq!(got, &bf(exp));
+          assert_eq!(got, &zc(exp));
           if let Some(last) = &self.last { assert!(got > last); }
         }
         self.last = got.clone();
@@ -962,6 +1038,15 @@ mod test {
     it.nxt(None);
     let mut it = mkr(None,Some("fvvq000000"),0).unwrap();
     it.nxt(None);
+
+    let mut it = mkr(Some("g0h8000000"),
+                     Some("g0h8000000_0000004000"), 2) .unwrap();
+    it.nxt(Some("g0h8000000_0000001ala"));
+    it.nxt(Some("g0h8000000_0000002lak"));
+    it.nxt(None);
+    
+    let mut it = mkr(None, Some("g0h8000000"), 0).unwrap();
+    it.nxt(None);    
   }
 
   #[test]
